// the song's artist
var artist;
// the song's title
var title;
// counter for the lyrics plugin ID (array index)
var pluginIdx = 0;

/**
  * creates a google feeling lucky url
  * some scripts do not provide an API nor a static file naming scheme (e.g. siteurl/artist-track-lyrics.html)
  *
  * NOTE: this does not use google directly as google forbids scripts to use the feeling lucky function :)
  *	  instead use 302found.appspo.com which works around that issue
  *
  * query: the string we're searching for (normal google syntax)
  */
var buildGoogleFeelingLuckyURL = function buildGoogleFeelingLuckyURLFunction(query)
{
	// don't use google itself (since it does not work) - but use 302found instead (workaround for some little problem :))
	return 'http://302found.appspot.com/?q=' + query;
}

/**
  * helper function which sets up all global variables
  * this also calls the first getNextLyrics() (next iterations of getNextLyrics() will be called from lyricsDownloaderOnFinish)
  *
  * NOTE: this function is called by Amarok directly (all parameters are passed automatically)
  *
  * artistName: the artist's name
  * songName: the song's name
  * url: Amarok already has a lyrics URL and just wants us to fetch the lyrics from there
  *
  */
function getLyrix(artistName, songName, url)
{
	artist = artistName;
	title = songName;
	// reset plugin ID -> this is important!
	pluginIdx = 0;

	showDebugInfo('getLyrix() - artistName:' + artist + ' songName: ' + title);

	// Amarok wants us to find lyrics from any site
	if (url == '')
	{
		getNextLyrix();
	}
	else
	{
		// TODO: amarok wants us to fetch lyrics from a specific URL
		showDebugInfo('getLyrix(): url is not empty! ' + url);
	}
}


/**
  * uses a lyrics site plugin to get the lyrics
  * this will call site-specific functions
  *
  * NOTE: getNextLyrix() is called from lyricsDownloaderOnFinish if the current lyrics
  *	  plugin did NOT find any lyrics (so getNextLyrics will use the next plugin)
  *
  * no arguments since this uses global variables
  */
var getNextLyrix = function getNextLyrix()
{
	if (pluginIdx < lyrixSites.length)
	{
		// info message
		Amarok.Window.Statusbar.shortMessage(XL_CHECKING_LYRICS_SITE + lyrixSites[pluginIdx] + '...');

		showDebugInfo('fetching lyrics from site: ' + lyrixSites[pluginIdx]);

		// lyrics plugin specific functions
		lyricsDownloadURL = eval(lyrixSites[pluginIdx] + 'GetURL');
		lyricsDownloaderOnFinish = eval(lyrixSites[pluginIdx] + 'OnFinish');
		var url = lyricsDownloadURL(artist, title);

		showDebugInfo('fetching lyrics from URL: ' + url.toString());

		// create Downloader instance which will call lyricsDownloadOnFinish (site specific!) after download finished
		new Downloader(url, lyricsDownloaderOnFinish);

		// we can increment pluginIdx here because getNextLyrics() will only be called if fetching lyrics with current plugin failed
		pluginIdx++;
	}
	else
	{
		Amarok.Window.Statusbar.shortMessage(XL_ALL_SITES_CHECKED);
	}
}

/**
  * converts plaintext lyrics to HTML lyircs
  *
  * lyrics: plaintext lyrics
  * songURL: URL to the lyrics for the current song
  * artistURL: URL to the artist of the current song
  *
  * NOTE: htmlContent inside this function: only append urlencoded values (you
  *		can use escape() to escape plain-text)!
  *
  * returns the HTML page as string
  */
function plainToHtmlLyrics(lyrics, songURL, artistURL)
{
	// lyrics content as HTML
	var htmlContent = '';

	// XHTML-header
	htmlContent += '%3C%21DOCTYPE%20html%20PUBLIC%20%22-//W3C//DTD%20XHTML%201.0%20Transitional';
	htmlContent += '//EN%22%20%22http%3A//www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd%22%3E%3C';
	htmlContent += 'html%20xmlns%3D%22http%3A//www.w3.org/1999/xhtml%22%20xml%3Alang%3D%22en%22%20';
	htmlContent += 'lang%3D%22en%22%20dir%3D%22ltr%22%3E%3Chead%3E%3Cmeta%20http-equiv%3D%22Content-';
	htmlContent += 'Type%22%20content%3D%22text/html%3B%20charset%3Dutf-8%22%20/%3E%3Ctitle%3E';

	htmlContent += escape(artist + ' ' + title + ' Lyrics');
	htmlContent += '%3C/title%3E%3C/head%3E%3Cbody%3E%3Ch3%3E%3Ca%20href%3D%22';
	htmlContent += escape(songURL) + '%22%3E' + escape(title);
	htmlContent += '%3C/a%3E';
	htmlContent += escape(' ' + XL_SONG_BY + ' ');
	htmlContent += '%3Ca%20href%3D%22' + escape(artistURL) + '%22%3E' + escape(artist);
	htmlContent += '%3C/a%3E%3C/h3%3E%3Cpre%3E';
	htmlContent += escape(lyrics);
	htmlContent += '%3C/pre%3E%3Chr/%3E';
	htmlContent += escape(XL_ADDITIONAL_INFO);
	htmlContent += '%3Cul%3E%3Cli%3E%3Cstrong%3Eurl%3A%20%3C/strong%3E%3Ca%20href%3D%22';
	htmlContent += escape(songURL);
	htmlContent += '%22%20title%3D%22url%22%3E';
	htmlContent += escape(songURL);
	htmlContent += '%3C/a%3E%3C/li%3E%3C/ul%3E%3C/body%3E%3C/html%3E';

	// return real HTML code
	return unescape(htmlContent);
}